' Steuerung einer Auswuchtmaschine V2.0
' Auswertung von 2 Sensoren zum dynamischen Auswuchten von Rotoren
' - 2. Sensor noch ohne Entlade-Schaltung am Eingang, ggf. durch Pin auf Output und LOW schalten lsen
'
' Hardwareausfhrung: Gnther Haagers Turbinenwuchtmaschine, http://www.technik-tuefteler.de/motor/turbine/turbineauswuchten.htm
' Grundlage der Statemachine frs Men: RoboterNETZ, http://www.rn-wissen.de/index.php/Bascom_State_Machine_Menu
'
' Aufbau:
' LCD-Anzeige (2x24 Zeichen) und Tastenmodul mit 5 Tasten (oben/unten/links/rechts/Mitte)
' Messverstrker mit Eingangs-Tiefpass und Messgleichrichter
'
' Funktionen:
' Menfhrung ber LCD
' PWM-Ausgang fr Antriebsmotor, Drehzahlvorwahl kann im EEPROM gespeichert werden
' Analogwert des Piezo-Sensors erfassen und qualitativ anzeigen
'     -> keine Absolutwerte der Spannung, nur die Samplewerte des ADC
' Ausgangskondensator des Messgleichrichters wird vor der Messung entladen, Anzeige durch blinkende LED
' 4 Messungen ohne/mit Testmasse (jeweils manuell ausgelst), danach Anzeige von Winkel und Masse der Auswuchtmasse
' Optional Aufteilung der resultierenden Masse auf zwei feste Winkel (Split-Massen)
' Angabe der Korrekturmasse als prozentualer Wert der Testmasse
' (hinterlagerte Mathematik siehe Ende dieser Datei)
' Optional Angabe der Korrekturmasse als Lngenangabe in [mm] fr Mannis Inbusschrauben
'
' Harald Sattler, www.harald-sattler.de/html/auswuchtmaschine.htm
'
$regfile = "m328pdef.dat"
$crystal = 8000000                                          ' used crystal frequency, interner RC Oszillator
' Achtung! Wenn $baud definiert wird, ist _immer_ der HW-UART eingeschaltet
' und die antsprechenden PortPins (pin2) auf Eingang bzw. (pin3) auf Ausgang geschaltet.
' Steuerung dieser Pins ber PortD.0 bzw. PortD.1 ist dann nicht mehr mglich!
' Auch der Arduino _Bootloader_ schaltet diese Funktion ein und verhindert damit die Nutzung als IO-pins
'$baud = 38400 ' used baud rate

$framesize = 64                                             'Stack
$swstack = 64
$hwstack = 64
'$sim                                                        'empty wait loops - faster simulation - not for real world!!

Config Clockdiv = 1                                         ' Code luft mit 8 MHz

' Versioninfo. Letzte Stelle wird automatisch beim Compilieren inkrementiert
' (dient mir als Zhler fr Flash-Vorgnge, deshalb nicht rcksetzen).
' Info: Nicht jeder Compile-Durchgang wird auch geflasht.
$version 2 , 0 , 355
Dim Buildinfo As String * 20
Buildinfo = Version(1)                                      ' enthlt Datum und Uhrzeit des Erstellzeitpunkts

' Dimensionierung einiger Variablen
Dim Lcd_textbuffer1 As String * 30
Dim Lcd_textbuffer2 As String * 30
Dim Tempstring As String * 30
Dim Vstring As String * 30
Dim Tslen As Byte
Dim Lcdpos As Byte
Dim Editpos As Byte
Dim Gradzahl As Single
Dim Repeat As Word
Dim Repeat1 As Word
Dim Disptime As Word
Dim Disptemp As Word


' Defines fr die Tasten
Const Key_buttons = 4                                       'Anzahl der Tasten
Dim Keycode_string As String * Key_buttons
'              Oben | Unten | Links | Rechts                 ' Enter-Taste (Mitte=6) fr Menu nicht verwendet
Keycode_string = "{011}{003}{010}{002}"                     ' {006} Mitte = Nothalt
Dim Keycode(key_buttons) As Byte At Keycode_string Overlay
Dim Lastkey As Byte
Const Key_null = 16                                         'keine Taste gedrckt
Oben Alias 11
Unten Alias 3
Links Alias 10
Rechts Alias 2
Mitte Alias 6                                               ' Mitte ist Nothalt fr Routinen mit Motorlauf

Const Repeat_delay = 15000                                  ' Schleifen bei Tastenabfrage bis Repeat einsetzt

'******** LCD Settings ********************************************************
'Pins des LCD-Modules festlegen und LCD initialisieren
Config Lcdpin = Pin , Db4 = Portd.4 , Db5 = Portd.5 , Db6 = Portd.6 , Db7 = Portd.7 , E = Portd.3 , Rs = Portd.2
Config Lcd = 24x2
Initlcd
Waitms 100
Cursor Off Noblink                                          'hide cursor
Cls

Gosub Defcharacters                                         ' eigene Zeichen definieren

'******** Joystick/Key Settings ***********************************************
' Tastatur
Config Portb = Output
Config Kbd = Portb , Debounce = 20 , Cols = 3               ' (5..8) Tasten-Keyboard
Dim Key As Byte                                             ' Tastencode

'******** Motorsteuerung Settings *********************************************
' PWM zu Fu erzeugen
' Achtung! Wenn oben $baud gesetzt ist, wird der HW-COM-Port scheinbar fest auf Comm gestellt,
'          die PWM-Ausgabe funktioniert dann nicht -> keine Comm ber Seriell versuchen ;-)
Config Portd = Output
Pwm_ausgang Alias Portd.0
Config Pwm_ausgang = Output

Dim Count_max As Word
Dim Pwm_step As Word
Dim Pwm_changed As Bit
Dim Last_top As Word
Dim Last_bot As Word
Dim Countertop As Word

Config Timer0 = Timer , Prescale = 8 , Clear_timer = 1 , Compare_a = Disconnect       ' Timer1 erzeugt die PWM fr den Motor
Stop Timer0                                                 ' noch keine PWM erzeugen, Motor aus
Pwm_ausgang = 0                                             ' Ausgang abschalten

' -----------------------------------------------------------------------------
' im EEPROM werden einige Werte permanent abgespeichert
'  -->> (Das muss noch berdacht werden, ich speichere ja Werte, die dann immer wieder berschrieben werden wrden...) <<--
'       (gilt fr Testwinkel und Testmasse, falls die nderbar gemacht werden und auf jeden Fall fr Drehzahlvorwahl)
Dim Pwm_puls_hi As Word , Pwm_puls_lo As Word
Dim Store_pwm As Eram Word                                  ' Drehzahlvorgabewert im EEPROM speichern
Dim Winkelwert As Word
Dim Store_winkel As Eram Word                               ' Winkelvorgabewert im EEPROM speichern
Dim Flaechenwert As Word
Dim Store_flaechen As Eram Word                             ' Flchenvorgabewert im EEPROM speichern

Dim Mess_eprom1 As Eram Word
Dim Mess_eprom2 As Eram Word
Dim Mess_eprom3 As Eram Word
Dim Mess_eprom4 As Eram Word
If Mess_eprom1 <> 13 Then Mess_eprom1 = 13                  ' einmalig beliebige Werte fr Test ins EEPROM schreiben
If Mess_eprom2 <> 11 Then Mess_eprom2 = 11
If Mess_eprom3 <> 45 Then Mess_eprom3 = 45
If Mess_eprom4 <> 6 Then Mess_eprom4 = 6

Dim Eetestmasse As Eram Single
Dim Testmasse As Single
Testmasse = Eetestmasse                                     ' Testmassse ist 10 mm Schraube, entspricht "Masse 1"
'If Testmasse <> 1 Then Eetestmasse = 1                      ' einmalig ins EEPROM schreiben

Dim Eetestwinkel As Eram Single
Dim Testwinkel As Single
Testwinkel = Eetestwinkel
'If Testwinkel <> 120 Then Eetestwinkel = 120                ' einmalig ins EEPROM schreiben   (Conrad: 120, Lyn: 240)

Dim Dummy1 As Eram Word                                     ' Abstand im EEPROM, damit man EESpeedup sieht ;-)

Dim Eespeedup As Eram Byte                                  ' Zeit fr Motorstart vor Messung
'Eespeedup = 5
Dim Speedup As Byte                                         ' Zeit fr Motorstart vor Messung
Speedup = Eespeedup
Dim Speedval As Byte                                        ' Hochlaufzeit in Sekunden

' Variablen zur Definition des Anzeigenamens
Dim Eenamestring As Eram String * 24
Dim Namestring As String * 24
Dim Namebyte(24) As Byte At Namestring(1) Overlay

'Namestring = Space(24)                                      ' EEPROM einmalig vorbelegen mit Blanks
'Eenamestring = Namestring

Dim Eeflip_masse As Eram Byte                               ' Winkel fr Ausgleichsmasse um 180 drehen

Dim Eesensors As Eram Byte                                  ' Anzahl verwendeter Piezo-Sensoren
Dim Sensors As Byte
'Eesensors = 2                                               ' einmalig setzen, spter per Men

' -----------------------------------------------------------------------------
' Grenzwerte und Festlegungen fr die PWM-Generierung
Countertop = 255
Count_max = 240
Pwm_step = 1
Last_top = Count_max - Pwm_step
Last_bot = 15

If Store_pwm = 65535 Or Store_pwm > Count_max Then          ' EEPROM noch leer (FFFF) -> Defaultwert laden
   Pwm_puls_hi = 100                                        ' Defaultwert, willkrlich gewhlt
Else
   Pwm_puls_hi = Store_pwm                                  ' Vorgabe aus EEPROM holen
End If

Winkelwert = Store_winkel                                   ' Winkelwert aus EEPROM laden
Flaechenwert = Store_flaechen                               ' Flaechenwert aus EEPROM laden


' PWM-Erzeugung, luft hier aber noch nicht los, TIMER0 steht noch auf STOP
Compare0a = Pwm_puls_hi                                     ' Startwert fr Hi-Impuls
Pwm_puls_lo = Countertop - Pwm_puls_hi                      ' Startwert fr Lo-Impuls

On Compare0a Pwm_irq
Enable Compare0a                                            ' Timer0 im CompareMatch Mode betreiben

Dim Pwm_hi As Bit                                           ' Merker fr PWM-Signal = Hi
Pwm_hi = 1

Enable Interrupts                                           ' IRQs freischalten

'******** allg Variablen ******************************************************
Dim A As Byte , B As Byte , C As Byte
Dim I As Byte , J As Byte
Dim V As Word , W As Word

'******** Sensor input Settings ***********************************************
Dim Analogin1 As Word
Dim Analogin2 As Word
Dim Analogmax1 As Word
Dim Analogmean1 As Word
Dim Analogmean2 As Word
Dim Analog_ruhe1 As Word
Dim Analog_ruhe2 As Word
Dim Analog_lauf1 As Word
Dim Analog_lauf2 As Word
Dim Ergebnis As Word
Dim Messwertindex As Byte
Dim Messwert(4) As Word                                     ' array
Dim Messung_done(4) As Byte
Dim Messung_shown(5) As Byte                                ' Merker fr Ausgabe auf dem Display
For A = 1 To 4                                              ' initialisieren von Variablen
   Messwert(a) = 0
   Messung_done(a) = 0
   Messung_shown(a) = 0
Next A
   Messung_shown(5) = 0

Dim Messwert_str As String * 10
Dim Messwert_len As Byte

' Variablen fr einfachen Lowpassfilter
Dim Filter_reg1 As Long                                     ' 32 bit
Dim Filter_temp1 As Long                                    ' 32 bit
Dim Filter_input1 As Integer                                ' 16 bit
Dim Filter_output1 As Integer                               ' 16 bit
Dim Filter_reg2 As Long                                     ' 32 bit
Dim Filter_temp2 As Long                                    ' 32 bit
Dim Filter_input2 As Integer                                ' 16 bit
Dim Filter_output2 As Integer                               ' 16 bit

' Ausgangskondensator des Messgleichrichters muss vor Messbeginn definiert entladen werden
Discharge_pin Alias Portc.2
Config Discharge_pin = Output

' Messwert hat Gleichspannungsoffset von ca. 2,5 V, Messwert ca. 0..900 mV berlagert
Config Pinc.0 = Input                                       ' Eingang vom Messgleichrichter 1
Config Pinc.3 = Input                                       ' Eingang vom Messgleichrichter 2
Config Adc = Single , Prescaler = Auto , Reference = Avcc


' Variablen fr die Berechung von Korrekturmasse und Winkel
Dim Rechenwert(4) As Single
Dim Xresult As Single
Dim Yresult As Single
' Zwischenwerte der Formeln
Dim Part1 As Single                                         ' 2 * F6^2
Dim Part2 As Single                                         ' F7^2
Dim Part3 As Single                                         ' F8^2
Dim Part4 As Single                                         ' 4 * F5^2
Dim Part5 As Single
Dim Part6 As Single                                         ' 1 - COS(deg2rad(F9))
Dim Part7 As Single
Dim Part8 As Single
Dim Rad120 As Single
Dim Unbalance_ratio As Single
Dim Korrekturmasse As Single
Dim Masse As String * 35
Dim Zwiwert As String * 35
Dim Korrekturwinkel As Single
Dim Winkelint As Integer
Dim Winkel As String * 8
Dim Flip_masse As Byte                                      ' Winkel fr Ausgleichsmasse um 180 drehen

' Variablen fr Winkel Splitten
Dim Winkel_array(10) As Single                              ' gemeinsames Array fr alle einstellbaren Winkel (2..9)
Dim Winkel_links As Single
Dim Winkel_rechts As Single
Dim Masse1 As Single
Dim Masse2 As Single
Dim Winkel1 As Single
Dim Winkel2 As Single

' Variablen fr die Anzeige einer "Uhr" im Sinne von "Bitte warten..."
Dim Timerchar As Byte
Dim Uhr As String * 3


'********* State Variables frs Men ******************************************
Dim State As Word                                           'aktueller State
Dim State_renew As Byte                                     'Flag
Dim State_gosub As Word                                     'aktuelles Unterprogramm

' Merker fr "State wurde aufgerufen" (verhindert endlos-Aktualisierung des LCD), wird nicht bei allen States bentigt
Dim State_exed(110) As Byte

'Initial state variables
State_renew = 1
State = Loadlabel(s10)                                      'Startbildschirm
Key = Key_null                                              'keine Taste gedrckt

'Tastaturtest, nur Debug
'Goto Tastencodes

'********** MAIN-Loop *********************************************************
Do
   'Meneintrag und Tastencodes finden und ggf. State wechseln
   'hier nur Tastendruck auswerten
   Gosub Change_state
   'Pointer nach Statuswechsel neu setzen
   If State_renew = 1 Then Gosub Change_state

   ' Legt fest, ob Ausgabe auf dem LCD vor Aufruf des SUB oder danach erfolgt
   Const Lcd_pre_code = 1                                   ' = 1 -> vorher, = 0 -> nachher

   ' Ausgabe auf LCD VOR dem zugehrigen Unterprogramm
   #if Lcd_pre_code = 1
   'LCD refresh wenn Men verndert
   If State_renew = 1 Then
      State_renew = 0
      Gosub Lcd_print
   End If
   #endif

   'Unterprogramm des aktuellen State anspringen
   LDS R31, {State_gosub+1}                                 'High; see Bascom-Doc Mixing ASM and BASIC
   LDS R30, {State_gosub}                                   'Low
   LSR R31                                                  'Division durch 2 (Alternativ ADR verwenden)
   ROR R30
   'Call zum Sub
   ICALL


   '---------------------------------------------------------------
   'place for your own code in main loop
   '---------------------------------------------------------------

   ' Ausgabe auf LCD NACH dem zugehrigen Unterprogramm (so war das Original)
   #if Lcd_pre_code = 0
   'LCD refresh wenn Men verndert
   If State_renew = 1 Then
      State_renew = 0
      Gosub Lcd_print
   End If
   #endif

   ' Tastatur abfragen
   Key = Getkbd()
   If Key <> Key_null Then                                  ' wenn Taste gedrckt
      While Key = Getkbd()                                  ' warten bis Taste wieder losgelassen wird
      Wend
   End If

 Loop
End                                                         ' Programmende

'********* State routines ***************************************************
'---------------------------------------------------------------
'Subroutine: Change_state
'Call from:  main loop
'Purpose:    Status der State Machine feststellen und ggf. Wechseln
'Result:     Pointer auf State / Variable State_renew
'---------------------------------------------------------------
Change_state:
    lds R8, {State}
    lds R9, {State + 1}
    For I = 1 To Key_buttons
      Read W
      If Key = Keycode(i) Then
        If W <> Loadlabel(null) Then
           State_renew = 1
           Key = Key_null                                   'reset key status after get a new state of state machine (prevent influence on GOSUBs)
           State = W
        End If
      End If
    Next I

    Read State_gosub                                        'Adresse des akt. Unterprogramms einlesen
    Read Lcd_textbuffer1                                    'read LCD text
    Read Lcd_textbuffer2                                    'read LCD text
Return

'---------------------------------------------------------------
'Subroutine: Change_state_by_sub
'Call from:  subroutine
'Purpose:    change state of state machine by a subbroutine
'---------------------------------------------------------------
Change_state_by_sub:
   State_renew = 1
   Gosub Change_state
Return

'---------------------------------------------------------------
'DATA:       State Machine
'Result:     Zeiger auf States, Unterprogramme
'            Mentexte
'Format:     1. Zeile: fr jede Taste genau eine Verzweigung ADR2
'            2. Zeile: Sprunglabel als ADR2 fr ein ggf. anzuspringendes Unterprogramm
'            3. Zeile: Displaytext 1. Zeile des akt. Status als DATA Feld
'            4. Zeile: Displaytext 2. Zeile des akt. Status als DATA Feld
'               (Die Texte knnen nur Konstanten sein, nicht zur Laufzeit berechenbar!! ... DATA)
'Hinweis:    wenn nichts passieren soll wird das Label NULL eingetragen
'---------------------------------------------------------------

Null:
         'Null is a dummy flag for State and Gosub -> do nothing
         Key = Key_null
         Return

'        Hoch (o)  : Runter (u): Links (l) :  Rechts (r)
S05:
         Adr2 Null : Adr2 S10 : Adr2 Null : Adr2 Null
         Adr2 Hilfe
         Data ""
         Data ""
'------------------------------------
S10:
         Adr2 S05 : Adr2 S15 : Adr2 Null : Adr2 S11         'Bedeutung der Zeilen:
         Adr2 Root                                          'Subroutine for current State
         Data ""                                            'Menue Display Text
         Data "    Auswuchtmaschine"                        'Menue Display Text
S11:
             Adr2 Null : Adr2 S12 : Adr2 S10 : Adr2 Null
             Adr2 Showversion
             Data ""
             Data ""
S12:
             Adr2 Null : Adr2 Null : Adr2 S10 : Adr2 S13
             Adr2 Testmessung
             Data "Analogwert einlesen"
             Data ""
S13:
             Adr2 Null : Adr2 Null : Adr2 S10 : Adr2 Null
             Adr2 Kreisberechnungen
             Data "Kreisberechnungen"
             Data ""
'------------------------------------
S15:
         Adr2 S10 : Adr2 S20 : Adr2 Null : Adr2 S16
         Adr2 Umess
         Data "Testlauf"
         Data "                       {006}"
S16:
             Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
             Adr2 Unwuchtmessung
             Data "PWM: xxx"
             Data "Unwucht: xxx           {007}"
'------------------------------------
S20:
         Adr2 S15 : Adr2 S50 : Adr2 Null : Adr2 S21
         Adr2 Null
         Data "Messung"
         Data "                       {006}"
S21:
             Adr2 S25 : Adr2 S31 : Adr2 S20 : Adr2 S22
             Adr2 Mtm_wo
             Data "Messung ohne Testmasse"
             Data "Start mit u            {043}"
S22:
             Adr2 Null : Adr2 S32 : Adr2 S21 : Adr2 S23
             Adr2 Mtm_w0
             Data "Messung Testmasse @ 0{223}"              ' ...@ 0
             Data "Start mit u            {003}"
S23:
             Adr2 Null : Adr2 S33 : Adr2 S22 : Adr2 S24
             Adr2 Mtm_w1
             Data "Messung Testmasse @"                     ' ...@ 120 auf Pos. 21 -> xxx  120{223}
             Data "Start mit u            {003}"
S24:
             Adr2 Null : Adr2 S34 : Adr2 S23 : Adr2 Null
             Adr2 Mtm_w2
             Data "Messung Testmasse @"                     ' ...@ 240 auf Pos. 21 -> xxx  240{223}
             Data "Start mit u            {004}"
S25:
                Adr2 Null : Adr2 Null : Adr2 S21 : Adr2 S26
                Adr2 Null
                Data "Messwerte l{239}schen?"
                Data "Best{225}tigung mit r      {253}"
S26:
                Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
                Adr2 Messwerte_geloescht
                Data "Messwerte gel{239}scht"
                Data ""
'------------------------------------
S30:
         Adr2 S10 : Adr2 S10 : Adr2 S10 : Adr2 S10
         Adr2 Null
         Data ""
         Data "                       {006}"
S31:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Messung1_31
            Data "Messung"
            Data "M1, ohne Testmasse"
S32:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Messung1_32
            Data "Messung"
            Data "M2, Testmasse @ 0{223}"
S33:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Messung1_33
            Data "Messung"
            Data "M3, Testmasse @"
S34:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Messung1_34
            Data "Messung"
            Data "M4, Testmasse @"
'------------------------------------
S40:
         Adr2 S10 : Adr2 S10 : Adr2 S10 : Adr2 S10
         Adr2 Null
         Data ""
         Data "                       {006}"
S41:
            Adr2 S21 : Adr2 S51 : Adr2 Null : Adr2 S22
            Adr2 Messwert1_41
            Data "  M1 |  M2 |  M3 |  M4"
            Data ""
S42:
            Adr2 S22 : Adr2 S51 : Adr2 Null : Adr2 S23
            Adr2 Messwert1_42
            Data "  M1 |  M2 |  M3 |  M4"
            Data ""
S43:
            Adr2 S23 : Adr2 S51 : Adr2 Null : Adr2 S24
            Adr2 Messwert1_43
            Data "  M1 |  M2 |  M3 |  M4"
            Data ""
S44:
            Adr2 S24 : Adr2 S51 : Adr2 Null : Adr2 Null
            Adr2 Messwert1_44
            Data "  M1 |  M2 |  M3 |  M4"
            Data ""
'------------------------------------
S50:
         Adr2 S20 : Adr2 S60 : Adr2 Null : Adr2 S51
         Adr2 Messergebnisse
         Data "Messergebnisse"
         Data "                       {006}"
S51:
            Adr2 Null : Adr2 S61 : Adr2 S50 : Adr2 S52
            Adr2 Messwerte_anzeigen
            Data "  M1 |  M2 |  M3 |  M4"
            Data "                       {003}"
S52:
               Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
               Adr2 Messwerte_edit
               Data ""
               Data ""
'------------------------------------
S60:
         Adr2 S50 : Adr2 S65 : Adr2 Null : Adr2 S61
         Adr2 Korrmasse
         Data "Korrekturmasse"
         Data "                       {006}"
S61:
            Adr2 S63 : Adr2 Null : Adr2 S60 : Adr2 S62
            Adr2 Berechnung
            Data "Korrekturmasse"
            Data "                       {253}"
S62:
            Adr2 Null : Adr2 Null : Adr2 S61 : Adr2 Null
            Adr2 Splitmassen
            Data "Split-Massen           {001}"
            Data ""
S63:
               Adr2 Null : Adr2 Null : Adr2 S61 : Adr2 S64
               Adr2 Demomabbruch
               Data "Demomassen vorgeben?"
               Data "Best{225}tigung mit r      {253}"
S64:
               Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
               Adr2 Demomassen
               Data "  M1 |  M2 |  M3 |  M4"
               Data ""
'------------------------------------
S65:
         Adr2 S60 : Adr2 Null : Adr2 Null : Adr2 S70
         Adr2 Einstellungen
         Data "Einstellungen"
         Data "                       {005}"
'------------------------------------
S70:
         Adr2 Null : Adr2 S75 : Adr2 S65 : Adr2 S71
         Adr2 Drehzahl
         Data "Drehzahlvorwahl"
         Data "                       {003}"
S71:
            Adr2 Null : Adr2 Null : Adr2 S70 : Adr2 Null
            Adr2 Drehzahl_change
            Data "Drehzahl {225}ndern mit o/u"
            Data "                       {043}"
S72:
            Adr2 Null : Adr2 Null : Adr2 S71 : Adr2 S73
            Adr2 Null
            Data "Drehzahl speichern?"
            Data "Best{225}tigung mit r      {253}"
S73:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Drehzahl_save
            Data "Drehzahl gespeichert"
            Data ""
'------------------------------------
S75:
         Adr2 S70 : Adr2 S80 : Adr2 S65 : Adr2 S76
         Adr2 Winkelsub
         Data "Winkel"
         Data "                       {043}"
S76:
            Adr2 Null : Adr2 Null : Adr2 S75 : Adr2 Null
            Adr2 Winkel_change
            Data "Winkel {225}ndern mit o/u"
            Data "                       {043}"
S77:
            Adr2 Null : Adr2 Null : Adr2 S76 : Adr2 S78
            Adr2 Null
            Data "Winkel speichern?"
            Data "Best{225}tigung mit r      {253}"
S78:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Winkel_save
            Data "Winkel gespeichert"
            Data ""
'------------------------------------
S80:
         Adr2 S75 : Adr2 S85 : Adr2 S65 : Adr2 S81
         Adr2 Flaechen
         Data "Fl{225}chenanzahl"
         Data "                       {043}"
S81:
            Adr2 Null : Adr2 Null : Adr2 S80 : Adr2 Null
            Adr2 Flaechen_change
            Data "Fl{225}chen {225}ndern mit o/u"
            Data "                       {043}"
S82:
            Adr2 Null : Adr2 Null : Adr2 S81 : Adr2 S83
            Adr2 Null
            Data "Fl{225}chen speichern?"
            Data "Best{225}tigung mit r      {253}"
S83:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Flaechen_save
            Data "Fl{225}chenanzahl "
            Data "gespeichert"
'------------------------------------
S85:
         Adr2 S80 : Adr2 S90 : Adr2 S65 : Adr2 S86
         Adr2 Motorhochlauf
         Data "Motorhochlauf"
         Data "                       {043}"
S86:
            Adr2 Null : Adr2 Null : Adr2 S85 : Adr2 Null
            Adr2 Motorhochlauf_change
            Data "Zeit {225}ndern mit o/u"
            Data "                       {043}"
S87:
            Adr2 Null : Adr2 Null : Adr2 S86 : Adr2 S88
            Adr2 Null
            Data "Zeit speichern?"
            Data "Best{225}tigung mit r      {253}"
S88:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Motorhochlauf_save
            Data "Motorhochlaufzeit"
            Data "gespeichert"
'------------------------------------
S90:
         Adr2 S85 : Adr2 S95 : Adr2 S65 : Adr2 S91
         Adr2 Null                                          'Name_def
         Data "Name definieren"
         Data "                       {043}"
S91:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Name_change
            Data "Wahl:o/u  Pos:r  Exit:l"
            Data "                       {043}"
S92:
            Adr2 Null : Adr2 Null : Adr2 S90 : Adr2 S93
            Adr2 Null
            Data "Name speichern?"
            Data "Best{225}tigung mit r      {253}"
S93:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Name_save
            Data "Name gespeichert"
            Data ""
'------------------------------------
S95:
         Adr2 S90 : Adr2 Null : Adr2 S65 : Adr2 S96
         Adr2 Flipmasse
         Data "Ausgleichsmasse um 180{223}"
         Data "versetzen   Status:    {007}"                ' user Char in Sub gendert!!
S96:
            Adr2 Null : Adr2 Null : Adr2 S95 : Adr2 S97
            Adr2 Null
            Data "Einstellung speichern?"
            Data "Best{225}tigung mit r      {253}"
S97:
            Adr2 Null : Adr2 Null : Adr2 Null : Adr2 Null
            Adr2 Flipmasse_save
            Data "Ausgleichsmasse um 180{223}"
            Data "versetzen gespeichert"
'------------------------------------



'********* LCD SUB routine ***************************************************
'---------------------------------------------------------------
'Subroutine: Lcd_print
'Call from:  anywhere
'Purpose:    gibt 2 Zeilen Lcd_textbuffer auf dem LCD-Display aus
'Result:     LCD
'---------------------------------------------------------------
Lcd_print:                                                  'Print lcd_textbuffer
   Cls
   Lcd Lcd_textbuffer1
   Lowerline
   Lcd Lcd_textbuffer2
   State_renew = 0
Return

'----------------------------------------------------------------------------------------------------------
' Men-Routinen Start
'----------------------------------------------------------------------------------------------------------
'********* SUB routines der Men States **************************************
Hilfe:
If State_exed(5) = 0 Then                                   ' S05
   Cls
   Lcd "L:{001} U:{008} L/U:{004} R/O:{005}"
   Locate 2 , 1
   Lcd "O/U:{002} O/U/L/R:{043} L/R:{253}"
   Locate 2 , 24
   Lcd "{008}"

   State_exed(5) = 1
   State_exed(10) = 0
End If
Return
'----------------------------------------------------------------
Root:                                                       ' S10
   ' 123456789012345678901234
   '      Die ultimative
   Namestring = Eenamestring
   Namestring = Trim(namestring)
   C = Len(namestring)                                      ' in der Mitte der Zeile platzieren
   C = C / 2
   C = 13 - C

   Locate 1 , C
   Lcd Namestring

   State_exed(11) = 0
   State_exed(20) = 0
   State_exed(5) = 0
   If State_exed(10) = 0 Then
      Waitms 2000
      State_exed(10) = 1
   End If

   Locate 2 , 24
   Lcd "{006}"
Return

'----------------------------------------------------------------
' Anzeige der VersionInfo
Showversion:                                                ' S11
If State_exed(11) = 0 Then
  Vstring = Version(2)
  Lcd_textbuffer1 = "Version " + Vstring

  Tempstring = "Build " + Buildinfo
  Tslen = Len(tempstring ) - 3                              ' Sekunden abschneiden
  Tempstring = Left(tempstring , Tslen )
  Lcd_textbuffer2 = Tempstring

  Gosub Lcd_print
  Locate 2 , 24
  Lcd "{001}"
  State_exed(11) = 1
End If
Return

'----------------------------------------------------------------
' Analogeingang auslesen, Test
Testmessung:                                                ' S12
Analogmax1 = 0
   Do
      Analogin1 = Getadc(0)
      If Analogin1 > Analogmax1 Then
         Analogmax1 = Analogin1
         Locate 2 , 15
         Lcd "Max: " ; Analogmax1
      End If

      Locate 2 , 1
      Lcd Analogin1 ; " "

      Key = Getkbd()
   Loop Until Key <> Key_null

   If Key = Rechts Then State = Loadlabel(s13)
   If Key = Oben Then State = Loadlabel(s11)

   State_exed(11) = 0
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
' Winkelberechnungen ausgeben, Test
Kreisberechnungen:                                          ' S13

Do                                                          ' wir bleiben whrend der Einstellerei immer in diesem State
   Key = Getkbd()
   While Key = Getkbd()                                     ' warten bis Taste wieder losgelassen wird
   Wend

   Cls

   If Key = Links Then Exit Do                              ' links = 10

   If Key = Rechts Then                                     ' rechts = 2
   End If

   If Key = Oben Then                                       ' oben = 3
      ' increment Gradzahl
      'If Gradzahl < 360 Then
      Gradzahl = Gradzahl + 5
      'Else Gradzahl = 0
   End If

   If Key = Unten Then                                      ' unten = 11
      ' decrement Gradzahl
      'If Gradzahl > 0 Then
      Gradzahl = Gradzahl - 5
      'Else Gradzahl = 359
   End If


   Rad120 = Deg2rad(gradzahl)                               ' radian Representation des Testmasse-Winkels

   ' Berechnung Xresult
   Xresult = Cos(rad120)                                    ' X-Wert

   ' Berechnung Yresult
   Yresult = Sin(rad120)                                    ' Y-Wert


   Locate 1 , 1                                             ' 1. Zeile
   Zwiwert = Str(xresult)
   Zwiwert = Left(zwiwert , 6)                              ' drei Nachkommastellen
   Lcd "X:" ; Zwiwert ;
   Zwiwert = Str(yresult)
   Zwiwert = Left(zwiwert , 6)                              ' drei Nachkommastellen
   Lcd " Y:" ; Zwiwert
   Locate 1 , 20                                            ' 2. Zeile
   Lcd Gradzahl

' Berechung des Winkels   Atn
   Part1 = Yresult / Xresult
   Part2 = Atn(part1)
   Korrekturwinkel = Rad2deg(part2)                         ' Umrechnung von Radian in Grad

   Winkelint = Korrekturwinkel
   Winkel = Str(winkelint)
   Locate 2 , 1
   Lcd "Atn:" ; Winkel

' Berechung des Winkels   Atn2
   Part2 = Atn2(yresult , Xresult)
   Korrekturwinkel = Rad2deg(part2)                         ' Umrechnung von Radian in Grad

   Winkelint = Korrekturwinkel
   Winkel = Str(winkelint)
   Locate 2 , 10
   Lcd "Atn2:" ; Winkel

   Winkelint = Korrekturwinkel
   Winkel = Str(winkelint)
'   C = Len(winkel)
'   C = 4 - C

   ' Werte ausgeben
   Locate 2 , 20                                            ' 2. Zeile
   Lcd Winkel ; "{223} "                                    '; "q:" ; Quadrant

   State_exed(61) = 1

Loop

State = Loadlabel(s10)
Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Umess:                                                      ' S15
   If State_exed(15) = 0 Then
      State_exed(15) = 1
   End If
   State_exed(16) = 0
Return

'----------------------------------------------------------------
Unwuchtmessung:                                             ' S16
Dim Timerlaeuft As Bit
Sensors = Eesensors                                         ' Anzahl der angeschlossenen Piezo-Sensoren aus dem EEPROM holen

' Motor luft noch nicht
For A = 0 To 5                                              ' Ausgangs-C des Messgleichrichters entladen
'   Set Discharge_pin
'   Waitms 10
'   Reset Discharge_pin
'   Waitms 100
      Config Portc.0 = Output                               ' Eingang vom Messgleichrichter 1
      Config Portc.3 = Output                               ' Eingang vom Messgleichrichter 2
      Reset Portc.0
      Reset Portc.3

      Waitms 10

      Config Pinc.0 = Input                                 ' Eingang vom Messgleichrichter 1
      Config Pinc.3 = Input                                 ' Eingang vom Messgleichrichter 2
Next A
Waitms 500                                                  ' Pause fr Analogkonverter

' Ruhewert ermitteln (sollte bei ca. 2,5 V liegen, also ADC-Wandler-Ausgang ca. 512)
Analogmean1 = Getadc(0)                                     ' Mittelwertspeicher vorbelegen
Analogmean2 = Getadc(3)                                     ' Mittelwertspeicher vorbelegen

For A = 0 To 7                                              ' 2000 Messwerte aufmitteln
   For V = 0 To 250
      Analogin1 = Getadc(0)                                 ' Mittelwertspeicher vorbelegen
      Analogin2 = Getadc(3)

      Analogmean1 = Analogmean1 + Analogin1                 ' Mittelwert1 bilden
      Analogmean1 = Analogmean1 / 2
      Analogmean2 = Analogmean2 + Analogin2                 ' Mittelwert2 bilden
      Analogmean2 = Analogmean2 / 2
   Next V
Next A
Analog_ruhe1 = Analogmean1                                  ' Eingangswert fr Differenzbildung
Analog_ruhe2 = Analogmean2                                  ' Eingangswert fr Differenzbildung

If State_exed(16) = 0 Then                                  ' TIMER0 nur starten, wenn er nicht schon luft
   Start Timer0                                             ' PWM-Ausgabe starten -> Motor ein
   State_exed(16) = 1
End If

Locate 1 , 6                                                ' PWM-Einstellung anzeigen
Lcd Pwm_puls_hi ; "  "

If Sensors <> 1 Then                                        ' dann nehme ich 2 an
   Locate 2 , 1                                             ' Unwuchten fr linken und rechten Sensor anzeigen
   Lcd "Unwucht L:      R: "                                ' Werte auf 12 und 20 ausgeben
End If


   Repeat = 0
   Filter_reg1 = 0
   Filter_reg2 = 0
'-------------------------------------------------------------------------------
   Do
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd() And Key <> Key_null           ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' wenn Taste lnger gedrckt -> repeat
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
'         Waitms 60                                          ' repeat rate
      End If

      Analogmax1 = 0
      Timerlaeuft = 0

      Analogin1 = Getadc(0)
      Analogin2 = Getadc(3)

' nach der Messung den Ausgangskondensator des Messgleichrichters entladen, weil sonst bei
' sinkender Drehzahl die maximale Ausgangsspannung sehr lange ansteht und Messfehler verursacht
'      Set Discharge_pin
'      Waitus 10
'      Reset Discharge_pin
      Config Portc.0 = Output                               ' Eingang vom Messgleichrichter 1
      Reset Portc.0
      Config Pinc.0 = Input                                 ' Eingang vom Messgleichrichter 1


      Config Portc.3 = Output                               ' Eingang vom Messgleichrichter 2
      Reset Portc.3
      Config Pinc.3 = Input                                 ' Eingang vom Messgleichrichter 2


'      If Analogin1 > Analogmax1 Then Analogmax1 = Analogin1

' Beim Versuch einen Motor auszuwuchten lief der Analogwert immer Richtung Null,
' egal wie unwuchtig der Motor ist. Scheinbar bgelt der Filter alles Richtung Null???
' Prfen!  -> Auf dem Auswuchtstand wird der Anzeigewert gehalten, luft NICHT gegen Null, also so OK.
      Const Filter_shift = 5

' v Tiefpass ------------------------------------
      Filter_input1 = Analogin1
      Filter_temp1 = Filter_reg1
      Shift Filter_temp1 , Right , Filter_shift , Signed
      Filter_reg1 = Filter_reg1 - Filter_temp1
      Filter_reg1 = Filter_reg1 + Filter_input1
      Filter_output1 = Filter_reg1
      Shift Filter_output1 , Right , Filter_shift , Signed

      Filter_input2 = Analogin2
      Filter_temp2 = Filter_reg2
      Shift Filter_temp2 , Right , Filter_shift , Signed
      Filter_reg2 = Filter_reg2 - Filter_temp2
      Filter_reg2 = Filter_reg2 + Filter_input2
      Filter_output2 = Filter_reg2
      Shift Filter_output2 , Right , Filter_shift , Signed
' ^ Tiefpass ------------------------------------

      If Sensors <> 1 Then                                  ' dann nehme ich 2 an
         Locate 2 , 20
         If Filter_output1 >= Analog_ruhe1 Then Filter_output1 = Filter_output1 - Analog_ruhe1 Else Filter_output1 = Analog_ruhe1 - Filter_output1
         Lcd Filter_output1 ; "  "

         Locate 2 , 12
         If Filter_output2 >= Analog_ruhe2 Then Filter_output2 = Filter_output2 - Analog_ruhe2 Else Filter_output2 = Analog_ruhe2 - Filter_output2
         Lcd Filter_output2 ; "  "
      Else
         Locate 2 , 10
         If Filter_output1 >= Analog_ruhe1 Then Filter_output1 = Filter_output1 - Analog_ruhe1 Else Filter_output1 = Analog_ruhe1 - Filter_output1
         Lcd Filter_output1 ; "  "
      End If



      If Key = Key_null Then Repeat = 0

      If Key = Links Then Exit Do                           ' Rechts = 2 , Links = 10

      If Key = Oben Then
         If Pwm_puls_hi <= Last_top Then
            Pwm_puls_hi = Pwm_puls_hi + Pwm_step
            Pwm_changed = 1
         End If
         If Pwm_puls_hi > Count_max Then Pwm_puls_hi = Count_max       ' Grenzwert absichern
         Pwm_puls_lo = Countertop - Pwm_puls_hi
      End If

      If Key = Unten Then
         If Pwm_puls_hi >= Last_bot Then
            Pwm_puls_hi = Pwm_puls_hi - Pwm_step
            Pwm_changed = 1
         End If
         If Pwm_puls_hi < 0 Then Pwm_puls_hi = 0            ' Grenzwert absichern
         Pwm_puls_lo = Countertop - Pwm_puls_hi
      End If

      If Pwm_changed = 1 Then                               ' genderten PWM-Wert ausgeben
         Locate 1 , 6
         Lcd Pwm_puls_hi ; "  "
         Pwm_changed = 0
      End If


'-------------------------------------
      Key = Getkbd()
      If Key = Mitte Then                                   ' emergency stop
         Stop Timer0                                        ' PWM aus
         Pwm_ausgang = 0                                    ' Port ausschalten

         Cls
         Locate 1 , 10
         Lcd "Nothalt"
         Locate 2 , 1
         Lcd "Weiter: Beliebige Taste"
         W = 0
         While Getkbd() = Mitte                             ' warte, bis Taste wieder losgelassen
         Wend
         While Getkbd() = Key_null
            Incr W
            If W = 20000 Then                               ' "Nothalt" blinkt
               Locate 1 , 10
               Lcd "       "
            End If
            If W = 40000 Then
               Locate 1 , 10
               Lcd "Nothalt"
               W = 0
            End If
         Wend
         While Getkbd() <> Key_null                         ' warte, bis Taste wieder losgelassen
         Wend

         Gosub Defcharacters                                ' Original Userzeichen wieder herstellen
         Cls

         State = Loadlabel(s10)
         Gosub Change_state_by_sub
         Return
      End If
'-------------------------------------

   Loop
'-------------------------------------------------------------------------------

   If Key = Links Then State = Loadlabel(s15)               ' Messschleife verlassen

   Stop Timer0                                              ' Motor anhalten
   Pwm_ausgang = 0

   State_exed(15) = 0

   Key = Key_null
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Messeinstieg:                                               ' S20
   If State_exed(20) = 0 Then
      State_exed(20) = 1
   End If
   State_exed(21) = 0
   State_exed(22) = 0
   State_exed(23) = 0
   State_exed(24) = 0
Return

'----------------------------------------------------------------
Mtm_wo:                                                     ' S21
   If State_exed(21) = 0 Then
      State_exed(21) = 1
      State_exed(22) = 0
   End If
Return

'----------------------------------------------------------------
Mtm_w0:                                                     ' S22
   If State_exed(22) = 0 Then
      State_exed(22) = 1
      State_exed(21) = 0
      State_exed(23) = 0
   End If
Return

'----------------------------------------------------------------
Mtm_w1:                                                     ' S23
   If State_exed(23) = 0 Then
      State_exed(23) = 1
      Locate 1 , 21
      Lcd Winkelwert ; "{223}"
      State_exed(22) = 0
      State_exed(24) = 0
   End If
Return

'----------------------------------------------------------------
Mtm_w2:                                                     ' S24
   If State_exed(24) = 0 Then
      State_exed(24) = 1
      Locate 1 , 20
      Lcd "-" ; Winkelwert ; "{223}"
      State_exed(23) = 0
   End If
Return

'----------------------------------------------------------------
' alte Messwerte lschen (vor erneuter Messung)
Messwerte_geloescht:                                        ' S26
   For A = 1 To 4                                           ' initialisieren
      Messwert(a) = 0
      Messung_done(a) = 0
   Next A

   Gosub Disp_uhr                                           ' Menanzeige 2 Sekunden stehen lassen

   State = Loadlabel(s21)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
' Durchfhrung der Messung fr jede Stufe
Messprogramm:                                               ' die eigentliche Messroutine, fr alle gleich
' Motor luft noch nicht
   For A = 0 To 5                                           ' Ausgangs-C des Messgleichrichters entladen
      Set Discharge_pin
      Waitms 10
      Reset Discharge_pin
      Waitms 100
   Next A
   Waitms 500                                               ' Pause fr Analogkonverter

' Ruhewert ermitteln (sollte bei ca. 2,5 V liegen, also ADC-Wandler-Ausgang ca. 512)
   Analogmean1 = Getadc(0)                                  ' Mittelwertspeicher vorbelegen

'   Gosub Defcharacters2                                     ' Zeiger fr stilisierte Uhr definieren
   Gosub Defcharacters3                                     ' Wurm fr stilisierte Uhr definieren

   For A = 0 To 7                                           ' 2000 Messwerte aufmitteln
      Locate 1 , 24
      Timerchar = A Mod 8
      If Timerchar = 0 Then Timerchar = 8
      Uhr = Chr(timerchar)
      Lcd Uhr

      For V = 0 To 250
         Analogin1 = Getadc(0)
         Analogmean1 = Analogmean1 + Analogin1              ' Mittelwert bilden
         Analogmean1 = Analogmean1 / 2
      Next V
   Next A
   Analog_ruhe1 = Analogmean1                               ' Eingangswert fr Differenzbildung

   Locate 1 , 1                                             ' Status anzeigen
   Lcd "Ruhewert"
   Locate 2 , 22                                            ' hinter dem Mentext den Wert ausgeben
   Lcd Analog_ruhe1

' Motor starten
   Start Timer0                                             ' Motor anlaufen lassen

   Gosub Disp_uhr                                           ' 2 Sekunden Pause zum Anzeigen
                                                             ' die der Motor schon mal mit zum Anlaufen nutzt :-)
   Gosub Defcharacters3                                     ' Wurm fr stilisierte Uhr definieren

   Locate 1 , 1                                             ' Status anzeigen
   Lcd "Motorhochlaufphase"

   Analogmean1 = Getadc(0)                                  ' Mittelwertspeicher vorbelegen

   Speedval = Speedup * 10
   For A = 0 To Speedval                                    ' warten bis Rotor Solldrehzahl erreicht hat (ggf.anpassen)
      Locate 1 , 24                                         ' Wert fr Speedup kommt aus dem EEPROM
      Timerchar = A Mod 8
      If Timerchar = 0 Then Timerchar = 8
      Uhr = Chr(timerchar)
      Lcd Uhr

      Waitms 20                                             '98  ' 95 statt 100, damit 10 Runden 1 Sekunde ergeben

      For V = 0 To 400                                      ' 2000 Messwerte aufmitteln
         Analogin1 = Getadc(0)
         Analogmean1 = Analogmean1 + Analogin1              ' Mittelwert bilden
         Analogmean1 = Analogmean1 / 2
      Next V

      Locate 2 , 22                                         ' hinter dem Mentext den Wert ausgeben
      If Analogmean1 >= Analog_ruhe1 Then Analogmean1 = Analogmean1 - Analog_ruhe1 Else Analogmean1 = Analog_ruhe1 - Analogmean1
      Lcd Analogmean1 ; "  "

'-------------------------------------
      Key = Getkbd()
      If Key = Mitte Then                                   ' emergency stop
         Stop Timer0                                        ' PWM aus
         Pwm_ausgang = 0                                    ' Port ausschalten

         Cls
         Locate 1 , 10
         Lcd "Nothalt"
         Locate 2 , 1
         Lcd "Weiter: Beliebige Taste"
         W = 0
         While Getkbd() = Mitte                             ' warte, bis Taste wieder losgelassen
         Wend
         While Getkbd() = Key_null
            Incr W
            If W = 20000 Then                               ' "Nothalt" blinkt
               Locate 1 , 10
               Lcd "       "
            End If
            If W = 40000 Then
               Locate 1 , 10
               Lcd "Nothalt"
               W = 0
            End If
         Wend
         While Getkbd() <> Key_null                         ' warte, bis Taste wieder losgelassen
         Wend

         Gosub Defcharacters                                ' Original Userzeichen wieder herstellen
         Cls

         State = Loadlabel(s10)
         Gosub Change_state_by_sub
         Return
      End If
'-------------------------------------
   Next A

   Analogmean1 = Getadc(0)                                  ' Mittelwertspeicher vorbelegen

   Locate 1 , 1                                             ' Status anzeigen
   Lcd "Messung l{225}uft     "

   ' Zeiger
   For A = 0 To 50
      Locate 1 , 24
      Timerchar = A Mod 8
      If Timerchar = 0 Then Timerchar = 8
      Uhr = Chr(timerchar)
      Lcd Uhr

      For V = 0 To 400                                      ' 2000 Messwerte aufmitteln
         Analogin1 = Getadc(0)
         If Analogin1 > Analogmax1 Then Analogmax1 = Analogin1       ' Maximalwert ermitteln

         Analogmean1 = Analogmean1 + Analogin1              ' Mittelwert bilden
         Analogmean1 = Analogmean1 / 2
      Next V

      Locate 2 , 22                                         ' hinter dem Mentext den Wert ausgeben
      If Analogmean1 >= Analog_ruhe1 Then Analogmean1 = Analogmean1 - Analog_ruhe1 Else Analogmean1 = Analog_ruhe1 - Analogmean1
      Lcd Analogmean1 ; "  "
'-------------------------------------
      Key = Getkbd()
      If Key = Mitte Then                                   ' emergency stop
         Stop Timer0                                        ' PWM aus
         Pwm_ausgang = 0                                    ' Port ausschalten

         Cls
         Locate 1 , 10
         Lcd "Nothalt"
         Locate 2 , 1
         Lcd "Weiter: Beliebige Taste"
         W = 0
         While Getkbd() = Mitte                             ' warte, bis Taste wieder losgelassen
         Wend
         While Getkbd() = Key_null
            Incr W
            If W = 20000 Then                               ' "Nothalt" blinkt
               Locate 1 , 10
               Lcd "       "
            End If
            If W = 40000 Then
               Locate 1 , 10
               Lcd "Nothalt"
               W = 0
            End If
         Wend
         While Getkbd() <> Key_null                         ' warte, bis Taste wieder losgelassen
         Wend

         Gosub Defcharacters                                ' Original Userzeichen wieder herstellen
         Cls

         State = Loadlabel(s10)
         Gosub Change_state_by_sub
         Return
      End If
'-------------------------------------
   Next A

   Locate 1 , 24                                            ' Uhranzeige lschen
   Lcd " "

   Gosub Defcharacters                                      'Original Userzeichen wieder herstellen

   ' Motor anhalten
   Stop Timer0                                              ' keine PWM erzeugen, Motor aus
   Pwm_ausgang = 0

   Ergebnis = Analogmean1
'   Analog_lauf1 = Analogmax1                                  ' Messwert fr Differenzbildung
'   Analog_lauf1 = Analogmean1                                 ' Messwert fr Differenzbildung

'   If Analog_lauf1 > Analog_ruhe1 Then                        ' so sollte das sein
'      Ergebnis = Analog_lauf1 - Analog_ruhe1                  ' Ruhewert abziehen und fr Ausgabe und Berechnung speichern
'   Else                                                     ' ist aber auch schon passiert, das Ergebnis in der Anzeige
'      Ergebnis = Analog_ruhe1 - Analog_lauf1                  ' ist dann > 65535, also negativ
'   End If

   ' Messwertindex wird im Aufruf gesetzt und bestimmt die durchgefhrte Messung
   ' (1= ohne Masse, 2= mit Testmasse auf 0, 3= mit Testmasse auf 120, 4= mit Testmasse auf 240)
   Messwert(messwertindex) = Ergebnis
   Messung_done(messwertindex) = 1

   Waitms 500
Return

'----------------------------------------------------------------
Messung1_31:                                                ' S31
  'Messung ohne Testmasse
   Messwertindex = 1                                        ' 1. Messung, ohne Testmasse

   If State_exed(31) = 0 Then
      State_exed(31) = 1
      State_exed(32) = 0
   End If

   Gosub Messprogramm                                       ' die eigentliche Messroutine, fr alle gleich

   State_exed(41) = 0
   State_exed(51) = 0

   State = Loadlabel(s41)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Messung1_32:                                                ' S32
  'Messung, Testmasse @ 0
   Messwertindex = 2                                        ' 2. Messung, Testmasse auf 0

   If State_exed(32) = 0 Then
      State_exed(32) = 1
      State_exed(31) = 0
      State_exed(33) = 0
   End If

   Gosub Messprogramm                                       ' die eigentliche Messroutine, fr alle gleich

   State_exed(42) = 0
   State_exed(51) = 0

   State = Loadlabel(s42)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Messung1_33:                                                ' S33
  'Messung, Testmasse @ 120
   Messwertindex = 3                                        ' 3. Messung, Testmasse auf 120

   If State_exed(33) = 0 Then
      State_exed(33) = 1
      Locate 2 , 17
      Lcd Winkelwert ; "{223}"
      State_exed(32) = 0
      State_exed(34) = 0
   End If

   Gosub Messprogramm                                       ' die eigentliche Messroutine, fr alle gleich

   State_exed(43) = 0
   State_exed(51) = 0

   State = Loadlabel(s43)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Messung1_34:                                                ' S34
  'Messung, Testmasse @ 240
   Messwertindex = 4                                        ' 4. Messung, Testmasse auf 240

   If State_exed(34) = 0 Then
      State_exed(34) = 1
      Locate 2 , 17
      Lcd "-" ; Winkelwert ; "{223}"
      State_exed(33) = 0
   End If

   Gosub Messprogramm                                       ' die eigentliche Messroutine, fr alle gleich

   State_exed(44) = 0
   State_exed(51) = 0
   State = Loadlabel(s44)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Messwert1_41:                                               ' S41
   If State_exed(41) = 0 Then
'      Locate 2 , 1
'      '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
'      '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
'      Lcd "  -  |  -  |  -  |  -  "

      A = 0
      Gosub Messwerte_zeile2                                ' ausgeben
      State_exed(41) = 1

      Locate 2 , 24
      Lcd "{006}"
   End If
   State_exed(51) = 0
Return

'----------------------------------------------------------------
Messwert1_42:                                               ' S42
   If State_exed(42) = 0 Then
'      Locate 2 , 1
'      '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
'      '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
'      Lcd "  -  |  -  |  -  |  -  "

      A = 1
      Gosub Messwerte_zeile2                                ' ausgeben
      State_exed(42) = 1

      Locate 2 , 24
      Lcd "{006}"
   End If
   State_exed(51) = 0
Return

'----------------------------------------------------------------
Messwert1_43:                                               ' S43
   If State_exed(43) = 0 Then
'      Locate 2 , 1
'      '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
'      '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
'      Lcd "  -  |  -  |  -  |  -  "

      A = 2
      Gosub Messwerte_zeile2                                ' ausgeben
      State_exed(43) = 1

      Locate 2 , 24
      Lcd "{006}"
   End If
   State_exed(23) = 0
   State_exed(51) = 0
Return

'----------------------------------------------------------------
Messwert1_44:                                               ' S44
   If State_exed(44) = 0 Then
'      Locate 2 , 1
'      '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
'      '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
'      Lcd "  -  |  -  |  -  |  -  "

      A = 3
      Gosub Messwerte_zeile2                                ' ausgeben
      State_exed(44) = 1

      Locate 2 , 24
      Lcd "{002}"
   End If
   State_exed(24) = 0
   State_exed(51) = 0
Return

'----------------------------------------------------------------
Messergebnisse:                                             ' S50
   State_exed(51) = 0
   State_exed(20) = 0
Return
'----------------------------------------------------------------
Messwerte_anzeigen:                                         ' S51
   If State_exed(51) = 0 Then
      Locate 2 , 1
      '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
      '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
      Lcd "  -  |  -  |  -  |  -  "

      For A = 0 To 3
         Gosub Messwert_z2                                  ' ausgeben
      Next A
      State_exed(51) = 1
   End If
Return

'----------------------------------------------------------------
' Messwerte manuell eingeben, keine Messung, Test
Messwerte_edit:                                             ' S52
   Locate 1 , 1
   Lcd "  M1 |  M2 |  M3 |  M4"

   For A = 0 To 3
      Messung_done(a + 1) = 1
      Gosub Messwert_z2                                     ' ausgeben
   Next A

   Lcdpos = 0
   Locate 2 , 3
   Cursor On

   Repeat = 0
   Repeat1 = 0

   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' Repeat_delay ist Const declariert
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Incr Repeat1
         If Repeat1 < 10 Then
            Waitms 60
         Else
            Waitms 10
         End If
      End If

      If Key = Key_null Then
         Repeat = 0
         Repeat1 = 0
      End If

      If Key = Links Then Exit Do                           ' links = 10

      If Key = Rechts Then                                  ' rechts = 2
         If Lcdpos < 3 Then Lcdpos = Lcdpos + 1 Else Lcdpos = 0
      End If

      If Key = Oben Then                                    ' oben = 3
         If Messwert(lcdpos + 1) < 65535 Then Messwert(lcdpos + 1) = Messwert(lcdpos + 1) + 1
      End If

      If Key = Unten Then                                   ' unten = 11
         If Messwert(lcdpos + 1) > 0 Then Messwert(lcdpos + 1) = Messwert(lcdpos + 1) - 1
      End If

      For A = 0 To 3
         Gosub Messwert_z2                                  ' ausgeben
      Next A

      Locate 2 , 24
      Lcd "{043}"

      Editpos = Lcdpos * 6
      Editpos = Editpos + 3
      Locate 2 , Editpos

   Loop

Cursor Off Noblink

State_exed(61) = 0

State = Loadlabel(s51)
Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
' Ausgabe der Messwerte in der zweiten LCD-Zeile
Messwert_z2:                                                ' Messwerte Zeile 2
   ' A = x (x = 0..3), wird im Aufruf gesetzt
   If Messung_done(a + 1) = 1 Then
      C = A * 6
      C = C + 3
      Messwert_str = Str(messwert(a + 1))                   ' im Ausgabefeld zentrieren
      Messwert_len = Len(messwert_str)
      Messwert_len = Messwert_len / 2
      C = C - Messwert_len
      C = C - 1                                             ' zwei Blanks vorne dran, lschen von Zehner- und Hunderterstellen
      Locate 2 , C
      Lcd " " ; Messwert(a + 1) ; " "
   End If
Return

'----------------------------------------------------------------
' Ausgabe der Messwerte in der zweiten LCD-Zeile
Messwerte_zeile2:                                           ' Messwerte Zeile 2
   'Locate 2 , 1
   '   "12345|12345|12345|12345"   Zeichenanzahl je Feld
   '   "1    |7    |13   |19   "   Startposition jedes Feldes im Display
   'Lcd "  -  |  -  |  -  |  -  "

   For V = 0 To 3
      C = V * 6
         If C > 0 Then
            Locate 2 , C
            Lcd "|"
         End If
      If Messung_done(v + 1) = 1 Then
         C = C + 3
         Messwert_str = Str(messwert(v + 1))                ' im Ausgabefeld zentrieren
         Messwert_len = Len(messwert_str)
         Messwert_len = Messwert_len / 2
         C = C - Messwert_len
         C = C - 1                                          ' zwei Blanks vorne dran, lschen von Zehner- und Hunderterstellen
         Locate 2 , C
         Lcd " " ; Messwert(v + 1) ; " "
      End If
   Next V
Return

'----------------------------------------------------------------
Korrmasse:                                                  ' S60
   State_exed(61) = 0
   State_exed(65) = 0
Return

'----------------------------------------------------------------
' Berechnung der Korrekturmasse und des Winkels aus den Messwerten der 4 Messungen
Berechnung:                                                 ' S61
If State_exed(61) = 0 Then
   State_exed(62) = 0
   For A = 1 To 4                                           ' Messwerte von WORD in Single konvertieren
      Rechenwert(a) = Messwert(a)
   Next A

' F5 - Rechenwert(1), F6 - Rechenwert(2), F7 - Rechenwert(3), F8 - Rechenwert(4)  (F5..8=siehe EXCEL-Sheet Balancing.xls)

   Testmasse = Eetestmasse                                  ' Werte aus EEPROM holen
'   Testwinkel = Eetestwinkel
   Winkelwert = Store_winkel                                ' aus Store_winkel (WORD) Testwinkel (Single) machen
   Testwinkel = Winkelwert

   Rad120 = Deg2rad(testwinkel)                             ' radian Representation des Testmasse-Winkels

' Berechnung Xresult
   Part1 = Rechenwert(2) ^ 2
   Part1 = Part1 * 2                                        ' 2 * F6^2
   Part2 = Rechenwert(3) ^ 2                                ' F7^2
   Part3 = Rechenwert(4) ^ 2                                ' F8^2
   Part4 = Rechenwert(1) ^ 2
   Part4 = Part4 * 4                                        ' 4 * F5^2
   Part5 = Cos(rad120)
   Part6 = 1 - Part5                                        ' 1 - COS(deg2rad(F9))

   Part7 = Part1 - Part2
   Part7 = Part7 - Part3                                    ' ((2*F6^2)-(F7^2)-(F8^2))

   Part8 = Part4 * Part6                                    ' ((4*F5^2)*(1-COS(deg2rad(F9)))

   Xresult = Part7 / Part8                                  ' X-Wert

' Berechnung Yresult
   Part5 = Sin(rad120)

   Part6 = Part2 - Part3
   Part7 = Part4 * Part5

   Yresult = Part6 / Part7                                  ' Y-Wert

' Berechung unbalance ratio                                  ' =1/WURZEL(F11^2+F12^2)
   Part1 = Xresult ^ 2
   Part2 = Yresult ^ 2
   Part3 = Part1 + Part2
   Part4 = Sqr(part3)
   Unbalance_ratio = 1 / Part4

' Berechnung der Korrekturmasse
   Korrekturmasse = Unbalance_ratio * Testmasse
   Masse = Str(korrekturmasse)
   Masse = Left(masse , 4)                                  ' 4: eine Vorkommastelle + Komma + zwei Nachkommastellen

' Berechung des Winkels
   Part2 = Atn2(yresult , Xresult)
   Korrekturwinkel = Rad2deg(part2)                         ' Umrechnung von Radian in Grad

   Flip_masse = Eeflip_masse                                ' im EEPROM gespeicherten Wert verwenden
   If Flip_masse = 1 Then                                   ' Korrekturmasse um 180 versetzen?
      Part1 = -1 * Sgn(korrekturwinkel)
      Part2 = 180 - Abs(korrekturwinkel)
      Korrekturwinkel = Part1 * Part2
   End If

   Winkelint = Korrekturwinkel                              ' Single -> Integer
   Winkel = Str(winkelint)

' Werte ausgeben
   Locate 2 , 1
   Lcd Masse ; " @ " ; Winkel ; "{223}"
   State_exed(61) = 1
End If

Return

'----------------------------------------------------------------
Splitmassen:                                                ' S62
   If State_exed(62) = 0 Then
      State_exed(62) = 1
      State_exed(61) = 0

      ' Array passend fllen -> 2: 180, 3: 120, 4: 90, 5: 72, 6: 60, 7:..., 8:..., 9: 40
      Winkel1 = 360 / Flaechenwert

      If Korrekturwinkel < 0 Then Winkel1 = Winkel1 * -1    ' negativer Korrekturwinkel
                                                                ' -> negative Arraywerte
      For A = 0 To 9
         Winkel_array(a + 1) = Winkel1 * A
      Next A

      If Korrekturwinkel >= 0 Then                          ' positiver Korrekturwinkel
         A = 1
         While Korrekturwinkel >= Winkel_array(a)
            Incr A
         Wend
      Else                                                  ' negativer Korrekturwinkel
         A = 1
         While Korrekturwinkel < Winkel_array(a)
            Incr A
         Wend
      End If

      B = A - 1
      Winkel_links = Korrekturwinkel - Winkel_array(b)      ' Richtung Loch beim kleineren Winkel
      Winkel1 = Winkel_array(b)
      Winkel_rechts = Winkel_array(a) - Korrekturwinkel     ' Richtung Loch beim greren Winkel
      Winkel2 = Winkel_array(a)

      ' F15 = Masse
      ' F21 = Winkel_links
      ' F22 = Winkel_rechts
      ' Masse1 = F15*((SIN(BOGENMASS(F22)))/(SIN(BOGENMASS(F21+F22))))
      Part1 = Deg2rad(winkel_rechts)
      Part2 = Winkel_links + Winkel_rechts
      Part3 = Deg2rad(part2)
      Part4 = Sin(part1)
      Part5 = Sin(part3)
      Masse1 = Korrekturmasse * Part4                       'Ergebnis der Split-Masse fr Loch 1
      Masse1 = Masse1 / Part5
      ' Masse2 = F15*((SIN(BOGENMASS(F21)))/(SIN(BOGENMASS(F21+F22))))
      Part1 = Deg2rad(winkel_links)
      Part4 = Sin(part1)
      Masse2 = Korrekturmasse * Part4                       'Ergebnis der Split-Masse fr Loch 2
      Masse2 = Masse2 / Part5

      ' Masse und Winkel fr 1. Loch
      Masse = Str(masse1)
      Masse = Left(masse , 4)                               ' 4: eine Vorkommastelle + Komma + zwei Nachkommastellen

      Winkelint = Winkel1                                   ' Single -> Integer
      Winkel = Str(winkelint)

      ' Debugausgaben
      '      Cls
      '      Locate 1 , 1
      '      Lcd "a:" ; A ; "/" ; Winkel_array(a) ; "/" ; Winkel_array(b)

      Locate 2 , 1
      Lcd Masse ; " @ " ; Winkel ; "{223}"                  ' Masse und Winkel fr Loch 1 ausgeben

      ' Masse und Winkel fr 2. Loch
      Masse = Str(masse2)
      Masse = Left(masse , 4)                               ' 4: eine Vorkommastelle + Komma + zwei Nachkommastellen

      Winkelint = Winkel2                                   ' Single -> Integer
      Winkel = Str(winkelint)

      Locate 2 , 13
      Lcd "|" ; Masse ; " @ " ; Winkel ; "{223}"            ' Masse und Winkel fr Loch 2 ausgeben

   End If
Return

'----------------------------------------------------------------
Demomabbruch:                                               ' S63
   State_exed(61) = 0
Return

'----------------------------------------------------------------
Demomassen:                                                 ' S64
   Locate 2 , 1
   Lcd "     |     |     |   "
   Messwert(1) = Mess_eprom1
   Messwert(2) = Mess_eprom2
   Messwert(3) = Mess_eprom3
   Messwert(4) = Mess_eprom4

   For A = 0 To 3
      Messung_done(a + 1) = 1

      C = A * 6
      C = C + 3
      Messwert_str = Str(messwert(a + 1))                   ' im Ausgabefeld zentrieren
      Messwert_len = Len(messwert_str)
      Messwert_len = Messwert_len / 2
      C = C - Messwert_len
      Locate 2 , C
      Lcd Messwert(a + 1)
   Next A

   State_exed(61) = 0
   State_exed(65) = 0
   State = Loadlabel(s61)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Einstellungen:                                              ' S65
   If State_exed(65) = 0 Then
      State_exed(65) = 1
      Gosub Defcharacters                                   ' User characters wieder herstellen nach S95
   End If                                                   ' dort wurde  chr(7) umdefiniert
Return
'----------------------------------------------------------------
Drehzahl:                                                   ' S70
   Stop Timer0                                              ' PWM-Ausgabe stoppen -> Motor aus
   Pwm_ausgang = 0
   State_exed(71) = 0
   State_exed(65) = 0
Return

'----------------------------------------------------------------
Drehzahl_change:                                            ' S71
If State_exed(71) = 0 Then                                  ' TIMER0 nur starten, wenn er nicht schon luft
   Start Timer0                                             ' PWM-Ausgabe starten -> Motor ein
   State_exed(71) = 1
End If

   Lowerline
   Lcd "Pwm hi/lo: " ; Pwm_puls_hi ; "/" ; Pwm_puls_lo


   Repeat = 0
   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' wenn Taste lnger gedrckt -> repeat
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Waitms 60                                          ' repeat rate
      End If

      If Key = Key_null Then Repeat = 0

      If Key = Rechts Or Key = Links Then Exit Do           ' Rechts = 2 , Links = 10

      If Key = Oben Then
         If Pwm_puls_hi <= Last_top Then
            Pwm_puls_hi = Pwm_puls_hi + Pwm_step
         End If
         If Pwm_puls_hi > Count_max Then Pwm_puls_hi = Count_max       ' Grenzwert sichern
         Pwm_puls_lo = Countertop - Pwm_puls_hi
      End If

      If Key = Unten Then
         If Pwm_puls_hi >= Last_bot Then
            Pwm_puls_hi = Pwm_puls_hi - Pwm_step
         End If
         If Pwm_puls_hi < 0 Then Pwm_puls_hi = 0            ' Grenzwert sichern
         Pwm_puls_lo = Countertop - Pwm_puls_hi
      End If

      Locate 2 , 12
      Lcd Pwm_puls_hi ; "/" ; Pwm_puls_lo ; " "
   Loop

   If Key = Rechts Then State = Loadlabel(s72)              ' Drehzahlvorgabe speichern?
   If Key = Links Then State = Loadlabel(s70)               ' Zurck ohne speichern

   Key = Key_null
   Gosub Change_state_by_sub

Return

'----------------------------------------------------------------
' Vorgabewert fr Drehzahl im EEPROM speichern
Drehzahl_save:                                              ' S73
   If Pwm_puls_hi <> Store_pwm Then                         ' im EEPROM nur speichern wenn Wert gendert
      Store_pwm = Pwm_puls_hi
   End If

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s71)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Winkelsub:                                                  ' S75
   State_exed(76) = 0
   State_exed(65) = 0
Return

'----------------------------------------------------------------
Winkel_change:                                              ' S76
'If State_exed(76) = 0 Then
'   State_exed(76) = 1
'End If

   Lowerline
   Lcd "Winkel: " ; Winkelwert


   Repeat = 0
   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' wenn Taste lnger gedrckt -> repeat
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Waitms 60                                          ' repeat rate
      End If

      If Key = Key_null Then Repeat = 0

      If Key = Rechts Or Key = Links Then Exit Do           ' Rechts = 2 , Links = 10

      If Key = Oben Then
         If Winkelwert <= 360 Then
            Winkelwert = Winkelwert + 1
         End If
         If Winkelwert > 360 Then Winkelwert = 360          ' Grenzwert sichern
      End If

      If Key = Unten Then
         If Winkelwert >= 1 Then
            Winkelwert = Winkelwert - 1
         End If
         If Winkelwert < 0 Then Winkelwert = 0              ' Grenzwert sichern
      End If

      Locate 2 , 9
      Lcd Winkelwert ; " "
   Loop

   If Key = Rechts Then State = Loadlabel(s77)              ' Winkelvorgabe speichern?
   If Key = Links Then State = Loadlabel(s75)               ' Zurck ohne speichern

   Key = Key_null
   Gosub Change_state_by_sub

Return

'----------------------------------------------------------------
' Vorgabewert fr Winkel im EEPROM speichern
Winkel_save:                                                ' S78
   If Winkelwert <> Store_winkel Then                       ' im EEPROM nur speichern wenn Wert gendert
      Store_winkel = Winkelwert
   End If

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s76)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Flaechen:                                                   ' S80
   State_exed(81) = 0
   State_exed(65) = 0
Return

'----------------------------------------------------------------
Flaechen_change:                                            ' S81
'If State_exed(81) = 0 Then
'   State_exed(81) = 1
'End If

   Lowerline
   Lcd "Fl{225}chen: " ; Flaechenwert


   Repeat = 0
   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' wenn Taste lnger gedrckt -> repeat
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Waitms 60                                          ' repeat rate
      End If

      If Key = Key_null Then Repeat = 0

      If Key = Rechts Or Key = Links Then Exit Do           ' Rechts = 2 , Links = 10

      If Key = Oben Then
         If Flaechenwert < 9 Then
            Flaechenwert = Flaechenwert + 1
         End If
         If Flaechenwert > 9 Then Flaechenwert = 9          ' Grenzwert sichern
      End If

      If Key = Unten Then
         If Flaechenwert > 2 Then
            Flaechenwert = Flaechenwert - 1
         End If
         If Flaechenwert < 2 Then Flaechenwert = 2          ' Grenzwert sichern
      End If

      Locate 2 , 10
      Lcd Flaechenwert ; " "
   Loop

   If Key = Rechts Then State = Loadlabel(s82)              ' Flchenvorgabe speichern?
   If Key = Links Then State = Loadlabel(s80)               ' Zurck ohne speichern

   Key = Key_null
   Gosub Change_state_by_sub

Return

'----------------------------------------------------------------
' Vorgabewert fr Flchen im EEPROM speichern
Flaechen_save:                                              ' S83
   If Flaechenwert <> Store_flaechen Then                   ' im EEPROM nur speichern wenn Wert gendert
      Store_flaechen = Flaechenwert
   End If

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s91)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Motorhochlauf:                                              ' S85
   State_exed(86) = 0
   State_exed(65) = 0
Return

'----------------------------------------------------------------
Motorhochlauf_change:                                       ' S86
'If State_exed(86) = 0 Then
'   State_exed(86) = 1
'End If

   Lowerline
   Lcd "Sekunden: " ; Speedup                               ' Wert ist Sekunden

   Repeat = 0
   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' wenn Taste lnger gedrckt -> repeat
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Waitms 60                                          ' repeat rate
      End If

      If Key = Key_null Then Repeat = 0

      If Key = Rechts Or Key = Links Then Exit Do           ' Rechts = 2 , Links = 10

      If Key = Oben Then
         If Speedup < 50 Then
            Speedup = Speedup + 1
         End If
         If Speedup > 50 Then Speedup = 50                  ' Grenzwert sichern
      End If

      If Key = Unten Then
         If Speedup > 1 Then
            Speedup = Speedup - 1
         End If
         If Speedup < 1 Then Speedup = 1                    ' Grenzwert sichern
      End If

      Locate 2 , 11
      Lcd Speedup ; " "
   Loop

   If Key = Rechts Then State = Loadlabel(s87)              ' Flchenvorgabe speichern?
   If Key = Links Then State = Loadlabel(s85)               ' Zurck ohne speichern

   Key = Key_null
   Gosub Change_state_by_sub

Return

'----------------------------------------------------------------
' Vorgabewert fr Motorstartzeit im EEPROM speichern
Motorhochlauf_save:                                         ' S88
   If Speedup <> Eespeedup Then                             ' im EEPROM nur speichern wenn Wert gendert
      Eespeedup = Speedup
   End If

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s86)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
' Anzeigename ndern (mal ein bisschen Spa zwischendurch muss erlaubt sein ;-)
Name_change:                                                ' S91

   Lcdpos = 1

   Namestring = Eenamestring
   Namestring = Trim(namestring)

   Locate 2 , 1
   Lcd Namestring

   Repeat = 0
   Repeat1 = 0
   Disptime = 0
   Disptemp = Repeat_delay / 5000

   'Cursor On
   Do                                                       ' wir bleiben whrend der Einstellerei immer in diesem State
      If Repeat = 0 Then
         Key = Getkbd()

         While Key = Getkbd()                               'And Key <> Key_null           ' warten bis Taste wieder losgelassen wird
            Incr Repeat
            If Repeat > Repeat_delay Then                   ' Repeat_delay ist Const declariert
               Exit While
            End If
         Wend
      Else
         Key = Getkbd()
         Incr Repeat1
         If Repeat1 < 10 Then
            Waitms 60
         Else
            Waitms 60                                       ' 10 fr schneller werden, ist hier unschn, daher auch 60
         End If
      End If

      If Key = Key_null Then
         Repeat = 0
         Repeat1 = 0
      End If
                                                                              ' 17
      Incr Disptime                                         ' "Wahl:o/u  Pos:r  Exit:l"
      If Disptime > Disptemp Then
         Locate 1 , 18
         Lcd "Exit:l "
         Disptime = 0
      End If

      If Key = Links Then Exit Do                           ' links = 10

      If Key = Rechts Then                                  ' rechts = 2
         If Lcdpos < 24 Then Lcdpos = Lcdpos + 1 Else Lcdpos = 1
      End If

      If Key = Oben Then                                    ' oben = 3
         If Namebyte(lcdpos) < 253 Then Namebyte(lcdpos) = Namebyte(lcdpos) + 1
         Locate 1 , 18
         Lcd "   0x" ; Hex(asc(namebyte(lcdpos))) ; " "
         Disptime = 0
      End If

      If Key = Unten Then                                   ' unten = 11
         If Namebyte(lcdpos) > 32 Then Namebyte(lcdpos) = Namebyte(lcdpos) - 1
         Locate 1 , 18
         Lcd "   0x" ; Hex(asc(namebyte(lcdpos))) ; " "
         Disptime = 0
      End If

      If Namebyte(lcdpos) < 32 Then Namebyte(lcdpos) = 32   ' einschrnken auf Blank bis...
      If Namebyte(lcdpos) > 253 Then Namebyte(lcdpos) = 253 ' ...letztes darstellbares Zeichen
                                                             ' (um  nutzen zu knnen. Siehe LCD Datenblatt)

      If Namebyte(24) = 32 Then                             ' Blank: Wenn noch nichts da steht, Navigator anzeigen
         Locate 2 , 24
         Lcd "{043}"
      End If

      Locate 2 , Lcdpos
      Lcd Chr(namebyte(lcdpos))
      Cursor On
      Locate 2 , Lcdpos

   Loop

   Cursor Off Noblink

   State_exed(90) = 0
   State_exed(65) = 0
   Key = Key_null

   State = Loadlabel(s92)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
' Vorgabewert fr Gertename im EEPROM speichern
Name_save:                                                  ' S93
   If Eenamestring <> Namestring Then
      Eenamestring = Namestring                             ' neuen Namen im EEPROM speichern
   End If

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s90)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------
Flipmasse:                                                  ' S95
   Locate 2 , 20
   Lcd Flip_masse

   Deflcdchar 7 , 224 , 228 , 228 , 255 , 224 , 228 , 224 , 224       ' fr diesen State neu definieren
   Locate 2 , 30

   State_exed(65) = 0
Return

'----------------------------------------------------------------
' Vorgabewert fr Ausgleichsmasse versetzen im EEPROM speichern
Flipmasse_save:                                             ' S97
   ' Ausgleichsmasse 180 Grad versetzen
   If Flip_masse = 0 Then Flip_masse = 1 Else Flip_masse = 0
   Eeflip_masse = Flip_masse                                ' im eeprom speichern

   Gosub Disp_uhr                                           ' Zeit fr Anzeige

   State = Loadlabel(s95)
   Gosub Change_state_by_sub
Return

'----------------------------------------------------------------------------------------------------------
' Men-Routinen zu Ende
'----------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------
' Bitte warten... anzeigen
Disp_uhr:
'   Gosub Defcharacters2                                     ' Zeiger fr stilisierte Uhr definieren
   Gosub Defcharacters3                                     ' Wurm fr stilisierte Uhr definieren

   For C = 1 To 2
      For A = 0 To 7
         Locate 1 , 24
         Timerchar = A Mod 8
         If Timerchar = 0 Then Timerchar = 8
         Uhr = Chr(timerchar)
         Lcd Uhr
         Waitms 67
      Next A
   Next C

   Gosub Defcharacters                                      ' Wurm fr stilisierte Uhr definieren

Return

'----------------------------------------------------------------
Tastencodes:
Do
   Key = Getkbd()
   If Key <> Lastkey Then
      Lastkey = Key
      Cls
      Locate 1 , 1
      Lcd Key
   End If
Loop

'********* IRQ Routinen ********************************
' Erzeugung des PWM-Signals fr den Motor
Pwm_irq:
   If Pwm_hi = 1 Then                                       ' PWM-Signal ist HI, also folgt jetzt LO
      Pwm_ausgang = 0
      Compare0a = Pwm_puls_lo                               ' Zeit fr LO-Signal
      Pwm_hi = 0
   Else                                                     ' PWM-Signal ist LO, also folgt jetzt HI
      Pwm_ausgang = 1
      Compare0a = Pwm_puls_hi                               ' Zeit fr HI-Signal
      Pwm_hi = 1
   End If
Return

'----------------------------------------------------------------
Defcharacters:
' Eigene Zeichen definieren zur Anzeige der mglichen Bewegungen im Men
' Chr(&Hfd) = Links Rechts(geteilt-Zeichen)                         ' {253} links rechts
' Chr(&H2B) = hoch runter links rechts (+)                          ' {043} hoch runter links rechts
Deflcdchar 0 , 224 , 228 , 224 , 245 , 228 , 228 , 224 , 224       ' {000} runter
Deflcdchar 1 , 224 , 228 , 224 , 253 , 224 , 228 , 224 , 224       ' {001} links
Deflcdchar 2 , 224 , 228 , 228 , 245 , 228 , 228 , 224 , 224       ' {002} hoch runter
Deflcdchar 3 , 224 , 228 , 224 , 255 , 228 , 228 , 224 , 224       ' {003} links rechts runter
Deflcdchar 4 , 224 , 228 , 224 , 253 , 228 , 228 , 224 , 224       ' {004} links runter
Deflcdchar 5 , 224 , 228 , 228 , 247 , 224 , 228 , 224 , 224       ' {005} rechts hoch
Deflcdchar 6 , 224 , 228 , 228 , 247 , 228 , 228 , 224 , 224       ' {006} hoch runter rechts
Deflcdchar 7 , 224 , 228 , 228 , 253 , 228 , 228 , 224 , 224       ' {007} hoch runter links
Locate 2 , 30
'Deflcdchar 5 , 224 , 228 , 224 , 247 , 228 , 228 , 224 , 224       ' {005} rechts runter

Return
'----------------------------------------------------------------
Defcharacters2:
' Eigene Zeichen definieren zur Anzeige einer stilisierten Timers (Zeiger)
' Uhr ganz oben
Deflcdchar 0 , 228 , 228 , 228 , 224 , 224 , 224 , 224 , 224
Deflcdchar 1 , 225 , 226 , 228 , 224 , 224 , 224 , 224 , 224
Deflcdchar 2 , 224 , 224 , 231 , 224 , 224 , 224 , 224 , 224
Deflcdchar 3 , 224 , 224 , 228 , 226 , 225 , 224 , 224 , 224
Deflcdchar 4 , 224 , 224 , 228 , 228 , 228 , 224 , 224 , 224
Deflcdchar 5 , 224 , 224 , 228 , 232 , 240 , 224 , 224 , 224
Deflcdchar 6 , 224 , 224 , 252 , 224 , 224 , 224 , 224 , 224
Deflcdchar 7 , 240 , 232 , 228 , 224 , 224 , 224 , 224 , 224

' Uhr in der Mitte
'Deflcdchar 0 , 224 , 224 , 228 , 228 , 228 , 224 , 224 , 224
'Deflcdchar 1 , 224 , 224 , 225 , 226 , 228 , 224 , 224 , 224
'Deflcdchar 2 , 224 , 224 , 224 , 224 , 231 , 224 , 224 , 224
'Deflcdchar 3 , 224 , 224 , 224 , 224 , 228 , 226 , 225 , 224
'Deflcdchar 4 , 224 , 224 , 224 , 224 , 228 , 228 , 228 , 224
'Deflcdchar 5 , 224 , 224 , 224 , 224 , 228 , 232 , 240 , 224
'Deflcdchar 6 , 224 , 224 , 224 , 224 , 252 , 224 , 224 , 224
'Deflcdchar 7 , 224 , 224 , 240 , 232 , 228 , 224 , 224 , 224

' durch das Ansprechen des Controllers geht er wieder in den Commando mode (normaler Modus)
Locate 2 , 30

Return
'----------------------------------------------------------------
Defcharacters3:
' Eigene Zeichen definieren zur Anzeige einer stilisierten Timers (Laufwurm)
' Uhr ganz oben
Deflcdchar 0 , 254 , 224 , 224 , 224 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 1 , 231 , 225 , 224 , 224 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 2 , 225 , 225 , 225 , 225 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 3 , 224 , 224 , 225 , 225 , 227 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 4 , 224 , 224 , 224 , 224 , 239 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 5 , 224 , 224 , 224 , 240 , 252 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 6 , 224 , 240 , 240 , 240 , 240 , 224 , 224 , 224       ' replace [x] with number (0-7)
Deflcdchar 7 , 248 , 240 , 240 , 224 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)

' Uhr in der Mitte
'Deflcdchar 0 , 224 , 224 , 254 , 224 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)
'Deflcdchar 1 , 224 , 224 , 231 , 225 , 224 , 224 , 224 , 224       ' replace [x] with number (0-7)
'Deflcdchar 2 , 224 , 224 , 225 , 225 , 225 , 225 , 224 , 224       ' replace [x] with number (0-7)
'Deflcdchar 3 , 224 , 224 , 224 , 224 , 225 , 225 , 227 , 224       ' replace [x] with number (0-7)
'Deflcdchar 4 , 224 , 224 , 224 , 224 , 224 , 224 , 239 , 224       ' replace [x] with number (0-7)
'Deflcdchar 5 , 224 , 224 , 224 , 224 , 224 , 240 , 252 , 224       ' replace [x] with number (0-7)
'Deflcdchar 6 , 224 , 224 , 224 , 240 , 240 , 240 , 240 , 224       ' replace [x] with number (0-7)
'Deflcdchar 7 , 224 , 224 , 248 , 240 , 240 , 224 , 224 , 224       ' replace [x] with number (0-7)

' durch das Ansprechen des Controllers geht er wieder in den Commando mode (normaler Modus)
Locate 2 , 30

Return
'----------------------------------------------------------------

'***************************************************************************************************************************
' Mathematik-Grundlagen, die Formeln stammen aus "Balancing.xls"

' F5     Messwert(0)
' F6     Messwert(1)
' F7     Messwert(2)
' F8     Messwert(3)
' F9     Winkel (120)  (erst mal fest vorgegeben, kann variabel gemacht werden)
' F11    X result
' F12    Y result
' F13    U/T ration of unbalance to test mass
' F14    Test mass
' F15    Mass to be added
' F16    Angle Of Addition
' F44    Winkelabstand der Splitmasse 1 zum errechneten Winkel
' F45    Winkelabstand der Splitmasse 2 zum errechneten Winkel

' X Result                                = ((2 * F6 ^ 2) -(f7 ^ 2) -(f8 ^ 2) ) /((4 * F5 ^ 2) *(1 -cos(bogenmass(f9))) )
' Y result                                = ( (F7^2) - (F8^2) ) / ( (4*F5^2) * (SIN(BOGENMASS(F9))) )
' U/T ratio of unbalance to test mass     = 1 / WURZEL(F11^2 + F12^2)
' Test mass                               = 1  (erst mal fest vorgegeben, kann variabel gemacht werden)
' Material to be removed or added         = F13*F14
' Angle of material addition              = GRAD( ARCTAN( F12 / F11) )

' Aufteilung auf zwei Massen
' Masse1                                  = F15*((SIN(BOGENMASS(F45)))/(SIN(BOGENMASS(F44+F45))))
' Masse2                                  = F15*((SIN(BOGENMASS(F44)))/(SIN(BOGENMASS(F44+F45))))
'***************************************************************************************************************************